import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/extensions/file_extension.dart';
import 'package:xflutter_cli/src/extensions/nullable_extensions.dart';
import 'package:xflutter_cli/src/models/flag_structure.dart';
import 'package:xflutter_cli/src/structure/config/router.dart';
import 'package:xflutter_cli/src/structure/config/themes.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/templates/config/singleton_config.dart';
import 'package:xflutter_cli/src/structure/templates/resources/colors.dart';
import 'package:xflutter_cli/src/structure/templates/resources/dimens.dart';
import 'package:xflutter_cli/src/structure/templates/resources/themes.dart';
import 'package:xflutter_cli/src/structure/templates/resources/transitions.dart';
import 'package:xflutter_cli/src/structure/templates/storage/storage.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/base_widgets.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/extensions.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/responsive_layouts.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/singleton.dart';

import '../structure/creators/i_creator.dart';
import '../structure/templates/router/router.dart';

enum CliFlags { responsive, singleton, theme, router, tablet }

enum FormType { email, password, confirm_password, text }

enum ModelType { response, entity }

List<CliFlags> flags = [];

const Map<String, CliFlags> screenFlags = {
  "singleton": CliFlags.singleton,
  "tablet": CliFlags.tablet,
  "responsive": CliFlags.responsive,
  "theme": CliFlags.theme,
  "router": CliFlags.router,
};

void getScreenFlags() {
  final args = cliArguments;
  if (args.length >= 3) {
    args.where((element) => element.contains("--")).forEach((element) {
      final flag = element.replaceAll("--", "");
      screenFlags[flag]?.let((it) => flags.add(it));
    });
  }
}

/// map contains every [CliFlags] with its Directories and files.
Map<CliFlags, FlagStructure> flagStructures = {
  CliFlags.singleton: FlagStructure(
    packages: [getIt, sharedPreferences],
    files: [
      FileStructure(
        path: configDir,
        name: "singleton_config",
        content: singletonConfig,
      ),
      FileStructure(
        path: singletonWidgetsDir,
        name: "singleton_widget",
        content: singletonWidget,
      ),
      FileStructure(
        path: singletonWidgetsDir,
        name: "singleton_instance",
        content: singletonInstance,
      ),
    ],
  ),
  CliFlags.responsive: FlagStructure(
    files: [
      FileStructure(
        path: coreResponsiveDir,
        name: "device_screen_type_enums",
        content: deviceScreenType,
      ),
      FileStructure(
        path: coreResponsiveDir,
        name: "orientation_layout",
        content: orientationLayout,
      ),
      FileStructure(
        path: coreResponsiveDir,
        name: "responsive_builder",
        content: responsiveBuilder,
      ),
      FileStructure(
        path: coreResponsiveDir,
        name: "screen_type_layout",
        content: screenTypeLayout,
      ),
      FileStructure(
        path: coreResponsiveDir,
        name: "sizing_information",
        content: sizingInformation,
      ),
      FileStructure(
        path: coreResponsiveDir,
        name: "responsive_utils",
        content: responsiveUtils,
      ),
    ],
  ),
  CliFlags.theme: FlagStructure(
    packages: [sharedPreferences],
    config: initializeMainTheme,
    files: [
      FileStructure(
        path: resourcesColorsDir,
        name: "colors",
        content: () {
          final colors = getColors();
          return designColors(colors);
        },
        condition: "$resourcesColorsDir/colors.dart".asFile().notExists,
      ),
      FileStructure(
        path: resourcesColorsDir,
        name: "colors_night",
        content: () {
          final colors = getColors(dark: true);
          return darkDesignColors(colors);
        },
        condition: "$resourcesColorsDir/colors_night.dart".asFile().notExists,
      ),
      FileStructure(
        path: resourcesDimensDir,
        name: "dimensions",
        content: () {
          final dimens = getDimens();
          return designDimens(dimens);
        },
        condition: "$resourcesDimensDir/dimensions.dart".asFile().notExists,
      ),
      FileStructure(
        path: resourcesThemesDir,
        name: "themes",
        content: lightTheme,
      ),
      FileStructure(
        path: resourcesThemesDir,
        name: "themes_night",
        content: darkTheme,
      ),
      FileStructure(
        path: resourcesThemesDir,
        name: "transitions",
        packages: [animations],
        content: transitions,
      ),
      FileStructure(
        path: storageDir,
        name: "storage",
        packages: [getIt],
        content: storageConfig,
      ),
      FileStructure(
        path: coreProviderDir,
        name: "theme_provider",
        packages: [provider],
        content: themeProvider,
      ),
      FileStructure(
        path: extensionsDir,
        name: "theme_extensions",
        content: themeExtensions,
      ),
      nullableExtensionFile,
      FileStructure(
        path: coreLayoutsDir,
        name: "theme_widget",
        content: themeWidget,
      ),
      baseScaffoldFile,
    ],
  ),
  CliFlags.router: FlagStructure(
    packages: [autoRoute],
    devPackages: [autoRouteGenerator, buildRunner],
    config: initializeScreenRoute,
    files: [
      FileStructure(
        path: routerDir,
        name: "app_router",
        content: router,
      ),
    ],
  ),
};
