import 'package:xflutter_cli/src/commands/i_command.dart';
import 'package:xflutter_cli/src/commands/i_flags.dart';
import 'package:xflutter_cli/src/extensions/path_extension.dart';
import 'package:xflutter_cli/src/models/flag_structure.dart';
import 'package:xflutter_cli/src/structure/config/curd.dart';
import 'package:xflutter_cli/src/structure/config/localization.dart';
import 'package:xflutter_cli/src/structure/creators/form_creator.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/config/freezed.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';
import 'package:xflutter_cli/src/structure/templates/localization/localization.dart';
import 'package:xflutter_cli/src/structure/templates/models/freezed/response.dart';
import 'package:xflutter_cli/src/structure/templates/viewmodels/viewmodels.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/screen_widgets.dart';

import '../structure/creators/i_creator.dart';
import 'i_terminal.dart';

enum CliOptions { screen, path, localization, reusable, form, model, crud }

const Map<String, CliOptions> cliOptions = {
  "screen": CliOptions.screen,
  "path": CliOptions.path,
  "localization": CliOptions.localization,
  // "crud": CliOptions.crud,
  "form": CliOptions.form,
  "model": CliOptions.model,
};

List<CliOptions> getCommandOptions() {
  List<CliOptions> options = [];
  cliArguments.where((element) => !element.contains("--") && element != CliOptions.path.name).forEach((element) {
    if (cliOptions[element] != null) {
      options.add(cliOptions[element]!);
    }
  });
  return options;
}

String fetchPath() {
  return cliArguments.firstWhere((element) => element.contains("path="), orElse: () => "").split("path=").last;
}

/// map contains every [CliOptions] with its Directories and files.
Future<Map<CliOptions, FlagStructure>> optionStructures() async {
  final path = fetchPath();
  final rootPath = path.split("/").last;
  String className = "";
  try {
    className = rootPath.capitalizeChar();
  } catch (_) {}

  return {
    CliOptions.localization: FlagStructure(
      packages: [easyLocalization],
      config: initializeMainEasyLocalization,
      message: 'Localizations 👍',
      files: [
        FileStructure(
          path: localizationDir,
          name: "en-US",
          extension: "json",
          content: localization,
        ),
      ],
    ),
    CliOptions.screen: FlagStructure(
      packages: [lazyEvaluation, liveData],
      config: () async {
        await createDir(widgetsDir(path));
        if (flags.contains(CliFlags.router)) await runInShell(generateCommand);
      },
      files: [
        FileStructure(
          path: "lib/$path",
          name: '${rootPath}_screen',
          content: () => singletonInstance(
            className: className,
            rootPath: rootPath,
          ),
        ),
        FileStructure(
          path: mobileDir(path),
          name: '${rootPath}_mobile_screen',
          content: () => singletonWidget(
            className: "${className}Mobile",
            rootPath: rootPath,
            singletonName: "${className}ViewModel",
          ),
          condition: () => flags.contains(CliFlags.responsive),
        ),
        FileStructure(
          path: tabletDir(path),
          name: '${rootPath}_tablet_screen',
          content: () => singletonWidget(
            className: "${className}Tablet",
            rootPath: rootPath,
            singletonName: "${className}ViewModel",
          ),
          condition: () => flags.contains(CliFlags.tablet),
        ),
        FileStructure(
          path: viewmodelDir(path),
          name: '${rootPath}_viewmodel',
          content: () => viewModel(className: className, rootPath: rootPath),
        ),
        FileStructure(
          path: viewmodelDir(path),
          name: '${rootPath}_params',
          content: () => viewModelParams(className: className),
        ),
        ...uiModelsFiles,
        ...loaderFiles,
        snackbarFile,
        nullableExtensionFile,
        customizedAnimatedWidgetFile,
      ],
    ),
    CliOptions.form: FlagStructure(
      config: () => FormCreator().create(path),
      files: [...uiTextFieldsFiles, ...uiButtonsFiles, scaleFadeWidgetFile],
    ),
    CliOptions.reusable: FlagStructure(
      files: reusableFiles,
    ),
    CliOptions.model: FlagStructure(
      packages: [freezedAnnotation, jsonSerializable, jsonAnnotation],
      devPackages: [freezed, buildRunner],
      config: () async {
        cliArguments.contains(CliCommands.generate.name) ? await initModels.call() : await initializeFreezed.call();
        await runInShell(generateCommand);
      },
      files: [
        FileStructure(
          path: "${modelsResponseDir}/base_response",
          name: "base_response",
          content: baseResponse,
        ),
      ],
    ),
    CliOptions.crud: FlagStructure(
      packages: [connectivityPlus, retrofit],
      devPackages: [retrofitGenerator, buildRunner],
      config: initCrud,
      files: [
        FileStructure(
          path: "${modelsResponseDir}/base_response",
          name: "base_response",
          content: baseResponse,
        ),
      ],
    ),
  };
}
