import 'dart:io';
import 'package:xflutter_cli/src/extensions/file_extension.dart';
import '../structure/packages.dart';

/// colors for command line.
/*
Black:   \x1B[30m
Red:     \x1B[31m
Green:   \x1B[32m
Yellow:  \x1B[33m
Blue:    \x1B[34m
Magenta: \x1B[35m
Cyan:    \x1B[36m
White:   \x1B[37m
Reset:   \x1B[0m
 */

const blackColor = "0";
const redColor = "1";
const greenColor = "2";
const yellowColor = "3";
const blueColor = "4";
const magentaColor = "5";
const cyanColor = "6";
const whiteColor = "7";

const String widgetTitle = """
/// Generated By XFlutter Cli.
/// 
/// more info: https://xflutter-cli.aghiadodeh.com""";

/// https://pub.dev/packages/{package}
const String getIt = "get_it^7.2.0";
const String dio = "dio^4.0.4";
const String retrofit = "retrofit^3.0.1";
const String http = "http^0.13.4";
const String rxdart = "rxdart";
const String jsonSerializable = "json_serializable^6.2.0";
const String jsonAnnotation = "json_annotation^4.5.0";
const String sharedPreferences = "shared_preferences^2.0.15";
const String pathProvider = "path_provider";
const String connectivityPlus = "connectivity_plus^2.3.0";
const String flutterSpinkit = "flutter_spinkit^5.1.0";
const String provider = "provider^6.0.3";
const String easyLocalization = "easy_localization^3.0.1";
const String flutterSvg = "flutter_svg";
const String transparentImage = "transparent_image";
const String cachedNetworkImage = "cached_network_image";
const String lazyEvaluation = "lazy_evaluation";
const String eventBus = "event_bus^2.0.0";
const String freezedAnnotation = "freezed_annotation^1.1.0";
const String secureStorage = "flutter_secure_storage^5.0.2";
const String formz = "formz^0.4.1";
const String liveData = "flutterx_live_data^1.5.5";
const String autoRoute = "auto_route^4.0.1";
const String floor = "floor^1.2.0";
const String animations = "animations^2.0.3";
const String shimmer = "shimmer";
const String imageFade = "image_fade";
const String implicitlyAnimatedList = "implicitly_animated_reorderable_list";
const String animatedInfiniteScrollPagination = "animated_infinite_scroll_pagination";
const String buildRunner = "build_runner^2.1.11";
const String autoRouteGenerator = "auto_route_generator^4.0.0";
const String retrofitGenerator = "retrofit_generator^3.0.1+1";
const String flutterLints = "flutter_lints";
const String freezed = "freezed^1.1.0";
const String floorGenerator = "floor_generator^1.2.0";

/// flutter generator command
const String generateCommand = "flutter pub run build_runner build --delete-conflicting-outputs";

/// add package to `pubspec.yaml`
Future<void> addPackage(String mPackage, {bool dev = false}) async {
  try {
    final split = mPackage.split("^");
    String package = split.first;
    String? version = split.length > 1 ? split[1] : null;
    String _package = terminalColor(label: package);
    stdout.writeln("\nChecking ($_package) package in pubspec.yaml...🤌");
    if (!pubspec.contains("$package:")) {
      stdout.writeln('adding ($_package) to your packages ⌛');
      if (version != null)
        await addPackageByVersion(package, version, dev);
      else
        await addPackageByTerminal(package, dev);
    } else {
      stdout.writeln("${terminalColor(label: "✔", color: greenColor)} ($_package) package found in pubspec.yaml\n");
    }
  } catch (error) {
    stderr.writeln(error);
    exit(2);
  }
}

Future<void> addPackageByVersion(String package, String version, bool dev) async {
  final dependencies = dev ? 'dev_dependencies' : 'dependencies';
  await writeFileAsString("pubspec.yaml", pubspec.replaceFirst("$dependencies:".trim(), "$dependencies:\n  $package: ^$version".trim()));
  await initPubspec();
  await runInShell("flutter pub get");
}

Future<void> addPackageByTerminal(String package, bool dev) async {
  if (dev) // dev_dependencies
    await runInShell("flutter pub add -d $package");
  else // dependencies
    await runInShell("flutter pub add $package");
}

Future<void> runInShell(String command, {List<String> args = const []}) async {
  stdout.writeln('Running ${terminalColor(label: command, color: magentaColor)}...⌛');
  final result = await Process.run(command, args, runInShell: true);
  stdout.write(result.stdout);
  stderr.write(result.stderr);
}

/// print welcome message.
bool welcomeBoard() {
  String content = '''
+---------------------------------------------------+
|           Welcome to the XFlutter CLI!            |
+---------------------------------------------------+
|     Tutorials: https://medium.com/@aghiadodeh     |
+---------------------------------------------------+
|          Do you want to continue? [y/n]           |
+---------------------------------------------------+\n''';

  stderr.write(content);

  final answer = stdin.readLineSync();
  final validator = answer?.toLowerCase() == 'y' || answer?.toLowerCase() == 'yes';

  return answer != null && validator;
}

/// exit cli
void messageAndExit({String? message}) {
  if (message != null) {
    stderr.writeln(terminalColor(label: message, color: redColor));
  } else {
    stderr.writeln('Command not available! 😅');
    stderr.writeln('try ${terminalColor(label: "xflutter_cli", color: yellowColor)} help for commands.');
  }
  exit(2);
}

/// make user confirm to continue
bool confirmCommand({String? message}) {
  String content = '''
+---------------------------------------------------+
+---------------------------------------------------+
|          Do you want to continue? [y/n]           |
+---------------------------------------------------+\n''';

  stderr.write(message ?? content);

  final answer = stdin.readLineSync();
  final validator = answer?.toLowerCase() == 'y' || answer?.toLowerCase() == 'yes';

  return answer != null && validator;
}

String terminalColor({required String label, String color = cyanColor}) => "\x1B[3${color}m${label}\x1B[0m";
