import 'dart:async';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/creators/i_flag_creator.dart';
import 'package:xflutter_cli/src/structure/creators/i_option_creator.dart';
import '../i_command.dart';
import '../i_flags.dart';
import '../i_options.dart';

class GenerateCommand implements ICommand {
  final String path;
  const GenerateCommand(this.path);

  @override
  FutureOr<void> execute() async {
    List<CliOptions> options = getCommandOptions();

    if (options.isEmpty) {
      messageAndExit(message: """
no options passed in command 😅\navailabe options:
- ${CliOptions.screen.name}
- ${CliOptions.localization.name}
- ${CliOptions.form.name}
- ${CliOptions.model.name}
""");
    }

    // tablet flag require responsive
    if (flags.contains(CliFlags.tablet) && !flags.contains(CliFlags.responsive)) {
      //   messageAndExit(message: "`tablet` flag require `responsive` flag🤦");
      flags.add(CliFlags.responsive);
    }
    if (flags.contains(CliFlags.responsive) && !flags.contains(CliFlags.singleton)) {
      flags.add(CliFlags.singleton);
    }

    if (options.length != 1) {
      messageAndExit(message: "cannot pass more than one option😅");
    }

    if (options.contains(CliOptions.form)) {
      if (!flags.contains(CliFlags.theme)) flags.add(CliFlags.theme);
      flags.removeWhere((element) => element.name == CliFlags.singleton);
    }

    if (screenFlags.isNotEmpty && options.contains(CliOptions.screen)) {
      if (path == "") {
        messageAndExit(message: "No `path` provided for screen❗");
      }
      final iFlagCreator = IFlagCreator();
      await iFlagCreator.create(flags);
    }
    //if (!options.contains(CliOptions.reusable)) options.add(CliOptions.reusable);
    final iOptionCreator = IOptionCreator();
    return await iOptionCreator.create(options, flags);
  }
}
