import 'dart:async';
import 'dart:io';
import 'package:xflutter_cli/src/commands/i_flags.dart';
import 'package:xflutter_cli/src/commands/i_terminal.dart';

import '../i_command.dart';
import '../i_options.dart';

class HelpCommand implements ICommand {
  @override
  FutureOr<void> execute() {
    String highlightColor = yellowColor;
    stdout.writeln('Usage: xflutter_cli <command>\n');

    stdout.writeln('Available commands:');
    stdout.writeln("""
- ${terminalColor(label: "generate", color: highlightColor)}: Generates and/or modifies files based on a schematic.
- ${terminalColor(label: "init", color: highlightColor)}: Initialize your project with necessary packages and reusable files.
""");
    stdout.writeln("""
generate options:
- (${terminalColor(label: CliOptions.screen.name)}): generate screen with state management. 😎

- (${terminalColor(label: CliOptions.localization.name)}): initialize (easy_localization) package in your app. 🌐

- (${terminalColor(label: CliOptions.form.name)}): generate form inside your screen widgets with inputs such as (email, password) with state management. 😏

- (${terminalColor(label: CliOptions.model.name)}): generate @freezed models from json file. 😍

""");
    stdout.writeln("""
generate flags:
- (${terminalColor(label: CliFlags.responsive.name)}): used with (${terminalColor(label: 'generate ${CliOptions.screen.name}', color: highlightColor)}), give you the ability to wrap tablet and mobile layouts in your screen (${terminalColor(label: "tablet screen will not generated", color: magentaColor)}).

- (${terminalColor(label: CliFlags.tablet.name)}): used with (${terminalColor(label: 'generate ${CliOptions.screen.name}', color: highlightColor)}), give you the ability to wrap tablet and mobile layouts in your screen (${terminalColor(label: "generate mobile and tablet screens", color: magentaColor)}).

- (${terminalColor(label: CliFlags.singleton.name)}): used with (${terminalColor(label: 'generate ${CliOptions.screen.name}', color: highlightColor)}), state management to manage variables between screen and children widgets.

- (${terminalColor(label: CliFlags.theme.name)}): used with (${terminalColor(label: 'generate ${CliOptions.screen.name}', color: highlightColor)}), initialize your global app theme.

- (${terminalColor(label: CliFlags.router.name)}): used with (${terminalColor(label: 'generate ${CliOptions.screen.name}', color: highlightColor)}), give you ability to manage navigation between screens, initialize (${terminalColor(label: 'auto_route')}) package and wrap the generated screen in app routes.
""");
    exit(0);
  }
}
