import 'dart:io';
import 'package:xflutter_cli/src/extensions/nullable_extensions.dart';

extension DirectoryUtils on Directory {
  Future<bool> createIfNotExists({String? name = null}) async {
    if (await exists()) {
      return false;
    } else {
      name?.let((it) => print('\nCreating $it directory ⌛...'));
      await create();
      return true;
    }
  }

  Future<void> createWithSubFolders() async {
    final split = path.split("/");

    for (var i = 0; i < split.length; i++) {
      String p = split.take(i + 1).join("/");
      await Directory(p).createIfNotExists(name: split[i]);
    }
  }

  Future<List<String>> contentsList() async {
    await createWithSubFolders();
    List contents = listSync();
    List<String> paths = [];
    for (var fileOrDir in contents) {
      if (fileOrDir is File) {
        paths.add(fileOrDir.path);
      } else if (fileOrDir is Directory) {
        paths.add(fileOrDir.path);
      }
    }
    return paths;
  }
}

extension DirectoryListUtils on List<Directory> {
  Future<bool> allExists({String? name = null}) async {
    bool isExists = true;
    final directories = this;
    for (var i = 0; i < directories.length; i++) {
      if (!await directories[i].exists()) {
        isExists = false;
        break;
      }
    }
    return isExists;
  }
}

Future<String> findBasePath({String path = "lib"}) async {
  String basePath = "";
  final libDir = Directory(path);

  if (await libDir.exists()) {
    basePath = libDir.absolute.path;
  } else {
    final res = await Directory(path).create(recursive: true);
    basePath = res.absolute.path;
  }

  return basePath.replaceAll("-", "_");
}
