import 'dart:io';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/directory_extensions.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/models/flag_structure.dart';

extension FileUtils on File {
  bool notExists() {
    return existsSync() == false;
  }
}

Future<void> createFile(FileStructure file) async {
  try {
    final execute = file.condition?.call() ?? true;
    if ("${file.path}/${file.name}.${file.extension}".asFile().notExists() && execute) {
      await Directory(file.path).createWithSubFolders();
      for (var i = 0; i < file.packages.length; i++) {
        await addPackage(file.packages[i]);
      }
      final f = await File("${file.path}/${file.name}.${file.extension}").create();
      if (file.content != null) {
        final writer = f.openWrite();
        writer.write(file.content?.call());
        writer.close();
      }

      final filename = "${terminalColor(label: "✔", color: greenColor)} (${terminalColor(label: '${file.name}.${file.extension}', color: yellowColor)}) ";
      final message = "file created successfully";
      print(filename + message);
    }
  } catch (error) {
    stderr.write('creating ${file.path}/${file.name}.${file.extension} failed!, Error: $error');
    exit(2);
  }
}

Future<String> readFileAsString(String path) async {
  final string = await File(path).readAsString();
  return string;
}

Future<void> writeFileAsString(String path, String content) async {
  await File(path).writeAsString(content);
}
