import 'package:xflutter_cli/src/commands/i_flags.dart';
import 'package:xflutter_cli/src/extensions/path_extension.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/text_fields.dart';

class FormzModel {
  final String fileName;
  final String Function() fileContent;
  final String className;
  final FormType type;
  const FormzModel({
    required this.fileName,
    required this.fileContent,
    required this.className,
    required this.type,
  });

  String get import => "'package:${packageName}/models/forms/$fileName.dart'";
  String get updateCallback => "${className.split('formz').last.firstLower()}Changed";

  String formField(String name) => """LiveDataTextField<${className}>(
  label: '${name}',
  liveData: viewModel.params.${name},
  onTextChanged: viewModel.${updateCallback},
  errorText: () => viewModel.params.${name}.value.error?.message,
  ${type == FormType.email ? "keyboardType: TextInputType.emailAddress," : ""}
  ${type == FormType.password || type == FormType.confirm_password ? "obscureText: true," : ""}
),""";
  String liveDataVariable(String name) => "final MutableLiveData<${className}> $name = MutableLiveData(initialValue: const ${className}.pure());\n";
}

Map<FormType, FormzModel> formzMap = {
  FormType.text: FormzModel(
    fileName: "formz_text",
    fileContent: formzTextInput,
    className: formzText,
    type: FormType.text,
  ),
  FormType.email: FormzModel(
    fileName: "formz_email",
    fileContent: formzEmailInput,
    className: formzEmail,
    type: FormType.email,
  ),
  FormType.password: FormzModel(
    fileName: "formz_password",
    fileContent: formzPasswordInput,
    className: formzPassword,
    type: FormType.password,
  ),
  FormType.confirm_password: FormzModel(
    fileName: "formz_confirm_password",
    fileContent: formzConfirmPasswordInput,
    className: formzConfirmPassword,
    type: FormType.confirm_password,
  ),
};
