import 'dart:io';

import 'package:xflutter_cli/src/extensions/directory_extensions.dart';
import 'package:xflutter_cli/src/extensions/file_extension.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

Future<void> initializeMainEasyLocalization() async {
  try {
    const initializeLocalization = "await EasyLocalization.ensureInitialized();";
    const importLocalization = "import 'package:easy_localization/easy_localization.dart';";
    const configFlutterBinding = "WidgetsFlutterBinding.ensureInitialized();";
    const configLocalization = """
EasyLocalization(
        supportedLocales: const [Locale('en', 'US')],
        path: 'assets/localization',
        fallbackLocale: const Locale('en', 'US'),
        child: const MyApp(),
      ),
""";

    /// initialize main.dart
    String main = await readFileAsString("lib/main.dart");

    main = main.replaceAll("void main() {", "void main() async {");
    if (!main.contains(initializeLocalization)) {
      main = main.replaceAll("void main() async {", "void main() async {\n  $initializeLocalization");
    }

    main = main.replaceAll(configFlutterBinding, "");
    main = main.replaceAll("void main() async {", "void main() async {\n  $configFlutterBinding");

    if (!main.contains(importLocalization)) {
      main = "$importLocalization\n$main";
    }
    if (!main.contains("const MyApp()")) {
      main = main.replaceAll("MyApp()", "const MyApp()");
    }
    if (!main.contains("EasyLocalization(")) {
      main = main.replaceAll("const MyApp(),", "const MyApp()").replaceAll("const MyApp()", configLocalization);
    }
    if (!main.contains("locale: context.locale,")) {
      main = main.replaceAll("home:", "locale: context.locale,\nhome:");
    }
    if (!main.contains("localizationsDelegates: context.localizationDelegates,")) {
      main = main.replaceAll("home:", "localizationsDelegates: context.localizationDelegates,\nhome:");
    }
    if (!main.contains("supportedLocales: context.supportedLocales,")) {
      main = main.replaceAll("home:", "supportedLocales: context.supportedLocales,\nhome:");
    }

    await writeFileAsString("lib/main.dart", main);

    await Directory("assets/images").createWithSubFolders();
    await Directory("assets/svgs").createWithSubFolders();

    /// initialize pubspec.yaml
    String pubspec = await readFileAsString("pubspec.yaml");
    pubspec = modifyPubSpec(pubspec, "assets/svgs/");
    pubspec = modifyPubSpec(pubspec, "assets/images/");
    pubspec = modifyPubSpec(pubspec, "assets/localization/");
    await writeFileAsString("pubspec.yaml", pubspec);
    await initPubspec();
  } catch (_) {}
}

String modifyPubSpec(String pubspec, String edition) {
  String string = pubspec;
  if (!string.contains(edition)) {
    string = string.replaceAll("# assets:", "assets:");
    string = string.replaceAll("assets:", """
assets:
    - ${edition}""");
  }

  return string;
}
