import 'package:xflutter_cli/src/commands/i_options.dart';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/extensions/file_extension.dart';
import 'package:xflutter_cli/src/extensions/path_extension.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';

Future<void> initializeScreenRoute() async {
  final path = fetchPath();
  final rootPath = path.split("/").last;
  final className = rootPath.capitalizeChar();

  String begin = "routes: <AutoRoute>[";

  String router = await readFileAsString("${routerDir}/app_router.dart");

  String imports = router.isNotEmpty ? router.replaceBetweenTwoWords(begin: "import", end: "\npart").replaceAll("\npart", "") : "";

  final newImport = "import 'package:${packageName}/$path/${rootPath}_screen.dart';\n";
  if (!imports.contains(newImport.trim())) {
    imports = "$newImport$imports";
  }

  String routes = router.replaceBetweenTwoWords(begin: begin, end: "]").replaceAll("]", "");

  bool isInitial = routes.contains("initial: true");
  final newRoute = "AutoRoute(path: '/${rootPath}', page: ${className}Screen, name: '${className}'${!isInitial ? ', initial: true' : ''}),";
  routes = routes.replaceAll(begin, "");
  if (!routes.contains(newRoute.trim())) {
    routes = "\n$newRoute$routes";
  }

  router = router.replaceBetweenTwoWords(begin: "import", end: "\npart", replace: "$imports\npart");

  router = router.replaceBetweenTwoWords(begin: begin, end: "]", replace: "routes: <AutoRoute>[$routes]");
  await writeFileAsString("${routerDir}/app_router.dart", router);

  /// initialize main.dart
  String main = await readFileAsString("lib/main.dart");
  if (!main.contains("import 'router/app_router.dart';")) {
    main = "import 'router/app_router.dart';\n$main";
  }

  if (!main.contains("MaterialApp.router(") && main.contains("MaterialApp(")) {
    main = main.replaceAll("MaterialApp(", """
MaterialApp.router(
  routerDelegate: appRouter.delegate(),
  routeInformationParser: appRouter.defaultRouteParser(),${main.contains(easyLocalization) ? '\nkey: ValueKey("\${context.locale}"),' : ''}""");
    if (main.contains("home:")) {
      main = main.replaceBetweenTwoWords(begin: "home:", end: ",", replace: "");
    }
    await writeFileAsString("lib/main.dart", main);
  }
}
