import 'dart:io';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/nullable_extensions.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/extensions/file_extension.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/templates/resources/colors.dart';
import 'package:xflutter_cli/src/structure/templates/resources/dimens.dart';

String _color = yellowColor;
Future<void> initializeMainTheme() async {
  String main = await readFileAsString("lib/main.dart");

  final importStorage = "import 'package:${packageName}/storage/storage.dart';";
  final importTheme = "import 'package:${packageName}/ui/resources/themes/themes.dart';";
  final importDarkTheme = "import 'package:${packageName}/ui/resources/themes/themes_night.dart';";
  final importThemeProvider = "import 'package:${packageName}/ui/core/providers/theme_provider.dart';";
  const importProvider = "import 'package:provider/provider.dart';";
  const configFlutterBinding = "WidgetsFlutterBinding.ensureInitialized();";
  const stoageConfig = "await AppStorage.storageConfig();";
  if (!main.contains(importStorage)) {
    main = "$importStorage\n$main";
  }
  if (!main.contains(importTheme)) {
    main = "$importTheme\n$main";
  }
  if (!main.contains(importDarkTheme)) {
    main = "$importDarkTheme\n$main";
  }
  if (!main.contains(importProvider)) {
    main = "$importProvider\n$main";
  }
  if (!main.contains(importThemeProvider)) {
    main = "$importThemeProvider\n$main";
  }
  if (!main.contains(stoageConfig)) {
    main = main.replaceAll("void main() {", "void main() async {");
    main = main.replaceAll("void main() async {", "void main() async {\n  $stoageConfig");
  }

  if (!main.contains(configFlutterBinding)) {
    main = main.replaceAll("void main() async {", "void main() async {\n  $configFlutterBinding");
  }

  if (!main.contains("const MyApp()")) {
    main = main.replaceAll("MyApp()", "const MyApp()");
  }
  if (!main.contains("MultiProvider(")) {
    main = main.replaceAll("const MyApp(),", "const MyApp()").replaceAll("const MyApp()", """
MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => ThemeProvider()),
      ],
      child: const MyApp(),
      ),""");
  } else {
    main.replaceAll("providers: [", """providers: [
      ChangeNotifierProvider(create: (_) => ThemeProvider()),""");
  }
  if (main.contains("theme: ThemeData")) {
    main = main.replaceBetweenTwoWords(
        begin: "theme: ThemeData", end: "),", replace: "themeMode: context.watch<ThemeProvider>().mode,${newLine}theme: lightTheme,");
  }
  if (!main.contains("darkTheme:")) {
    main = main.replaceAll("theme: lightTheme,", "theme: lightTheme,${newLine}darkTheme: darkTheme,");
  }
  await writeFileAsString("lib/main.dart", main);
}

/// config App Theme colors by user from terminal
DesignColors getColors({bool dark = false}) {
  stdout.writeln("\n${dark ? 'Dark' : 'Light'} theme colors: 🎨");

  var primaryColor = "#6200EE";
  var secondaryBackgroundColor = dark ? "#222222" : "#F3F3F3";
  var secondaryColor = "#BB86FC";
  var textColor = dark ? "#FFFFFF" : "#333333";

  String mode = dark ? " Dark mode" : "";

  stdout.writeln('\nEnter Your$mode primaryColor, default is :${terminalColor(label: primaryColor, color: _color)}');
  primaryColor = stdin.readLineSync().defaultValue(primaryColor);

  stdout.writeln('Enter Your$mode secondaryBackgroundColor, default is :${terminalColor(label: secondaryBackgroundColor, color: _color)}');
  secondaryBackgroundColor = stdin.readLineSync().defaultValue(secondaryBackgroundColor);

  stdout.writeln('Enter Your$mode secondaryColor, default is :${terminalColor(label: secondaryColor, color: _color)}');
  secondaryColor = stdin.readLineSync().defaultValue(secondaryColor);

  stdout.writeln('Enter Your$mode textColor, default is :${terminalColor(label: textColor, color: _color)}');
  textColor = stdin.readLineSync().defaultValue(textColor);

  final colors = DesignColors(
    primaryColor: primaryColor.asColor(),
    secondaryBackgroundColor: secondaryBackgroundColor.asColor(),
    secondaryColor: secondaryColor.asColor(),
    textColor: textColor.asColor(),
  );

  stdout.writeln(colors.toString());
  stdout.writeln("\nYou can change these colors at any time from: '${terminalColor(
    label: "lib/ui/resources/${dark ? 'colors_night.dart' : 'colors.dart'}",
    color: yellowColor,
  )}'");
  return colors;
}

/// config App Dimens by user from terminal
Dimens getDimens() {
  String buttonCornerRadius = "24";
  String textFormFieldRadius = "6";
  String bottomSheetRadius = "36";

  stdout.writeln('Enter Your buttonCornerRadius, default is :${terminalColor(label: buttonCornerRadius, color: _color)}');
  buttonCornerRadius = stdin.readLineSync().defaultValue(buttonCornerRadius);

  stdout.writeln('Enter Your textFormFieldRadius, default is :${terminalColor(label: textFormFieldRadius, color: _color)}');
  textFormFieldRadius = stdin.readLineSync().defaultValue(textFormFieldRadius);

  stdout.writeln('Enter Your bottomSheetRadius, default is :${terminalColor(label: bottomSheetRadius, color: _color)}');
  bottomSheetRadius = stdin.readLineSync().defaultValue(bottomSheetRadius);
  final dimens = Dimens(
    buttonCornerRadius: buttonCornerRadius.asDouble(),
    textFormFieldRadius: textFormFieldRadius.asDouble(),
    bottomSheetRadius: bottomSheetRadius.asDouble(),
  );
  stdout.writeln(dimens.toString());
  stdout.writeln("\nYou can change these dimensions at any time from: '${terminalColor(label: "lib/ui/resources/dimensions.dart", color: yellowColor)}'");
  return dimens;
}
