import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/models/flag_structure.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';
import 'package:xflutter_cli/src/structure/templates/config/ui_config.dart';
import 'package:xflutter_cli/src/structure/templates/events/event_bus.dart';

import '../../extensions/file_extension.dart';

Future<void> initializeMainConfig() async {
  await addPackage(eventBus);

  await createFile(
    FileStructure(
      path: configDir,
      name: "ui_config",
      content: uiConfig,
    ),
  );
  await createFile(
    FileStructure(
      path: coreEventsDir,
      name: "bus_events",
      content: eventBusConfig,
    ),
  );

  const imports = "import 'config/ui_config.dart';\nimport 'ui/core/events/bus_events.dart';";

  String main = await readFileAsString("lib/main.dart");

  if (!main.contains(imports)) {
    main = "${imports}\n${main}";
  }

  // convert to StatefulWidget
  if (main.contains("MyApp extends StatelessWidget")) {
    main = main.replaceAll("const MyApp({Key? key}) : super(key: key);", "");
    main = main.replaceAll("class MyApp extends StatelessWidget {", """class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {

  @override
  void initState() {
    super.initState();
    // TODO: uiConfig(); // uncomment this method to config ui such as setup app Orientation (portrait and landscape)
    _handleSoftkeyboardEvents();
    _handleUnauthorizedEvent();
  }

  /// hide soft keyboard when (non-context class) emit event
  _handleSoftkeyboardEvents() {
    eventBus.on<SoftKeyboardEvent>().listen((event) {
      hideSoftKeyboard(context);
    });
  }

  /// logOut
  _handleUnauthorizedEvent() {
    eventBus.on<UnauthorizedEvent>().listen((event) async {
      // TODO: handle Unauthorized Event
    });
  }""");
  }

  if (!main.contains("GestureDetector")) {
    String mainWidget = main.replaceBetweenTwoWords(begin: "MaterialApp", end: ");").replaceAll(";", ",");
    mainWidget = """GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: () => hideSoftKeyboard(context),
      child: ${mainWidget}
    );""";

    main = main.replaceBetweenTwoWords(begin: "MaterialApp", end: ");", replace: mainWidget);
  }

  await writeFileAsString("lib/main.dart", main);
}
