import 'dart:io';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/extensions/directory_extensions.dart';
import 'package:xflutter_cli/src/models/flag_structure.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/tab_layout.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/text_fields.dart';
import 'package:xflutter_cli/src/structure/templates/widgets/textview.dart';
import '../templates/models/ui_model.dart';
import '../templates/widgets/animation_widget.dart';
import '../templates/widgets/base_widgets.dart';
import '../templates/widgets/buttons.dart';
import '../templates/extensions/common_widget.dart';
import '../templates/widgets/extensions.dart';
import '../templates/widgets/infinite_scroll_pagination.dart';

Future<void> createDir(String path) async {
  await Directory(path).createWithSubFolders();
}

Future<void> addPackages(List<String> packages, {bool dev = false}) async {
  for (var i = 0; i < packages.length; i++) {
    await addPackage(packages[i], dev: dev);
  }
}

FileStructure get nullableExtensionFile => FileStructure(
      path: extensionsDir,
      name: "nullable_extensions",
      content: nullableExtensions,
    );

FileStructure get customizedAnimatedWidgetFile => FileStructure(
      path: animationsWidgetsDir,
      name: "customized_animated_widget",
      content: customizedAnimatedWidget,
    );

FileStructure get snackbarFile => FileStructure(
      path: uiWidgetsDir,
      name: "snackbar",
      content: snackBar,
    );

FileStructure get baseScaffoldFile => FileStructure(
      path: coreLayoutsDir,
      name: "base_scaffold",
      content: baseScaffold,
    );
FileStructure get scaleFadeWidgetFile => FileStructure(
      path: animationsWidgetsDir,
      name: "scale_fade_widget",
      content: scaleFadeWidget,
    );

List<FileStructure> baseLayoutFiles = [
  baseScaffoldFile,
  FileStructure(
    path: coreLayoutsDir,
    name: "base_scroll_view",
    content: baseScrollView,
  ),
];

List<FileStructure> loaderFiles = [
  FileStructure(
    path: loaderWidgetsDir,
    name: "loader",
    content: loader,
    packages: [flutterSpinkit],
  ),
  FileStructure(
    path: loaderWidgetsDir,
    name: "live_data_loader",
    content: liveDataLoader,
    packages: [liveData],
  ),
  FileStructure(
    path: loaderWidgetsDir,
    name: "blur_widget",
    content: blurWidget,
  ),
  FileStructure(
    path: loaderWidgetsDir,
    name: "full_screen_loader",
    content: fullScreenLoader,
  ),
  FileStructure(
    path: loaderWidgetsDir,
    name: "progress_bar",
    content: progressBar,
  ),
];

List<FileStructure> uiModelsFiles = [
  FileStructure(
    path: uiModelsDir,
    name: "ui_message",
    content: uiMessage,
  ),
];

List<FileStructure> uiTextFieldsFiles = [
  FileStructure(
    path: formsWidgetsDir,
    name: "customized_text_form_field",
    content: textField,
  ),
  FileStructure(
    path: extensionsDir,
    name: "formz_extensions",
    content: formzExtensions,
    packages: [formz],
  ),
  FileStructure(
    path: formsWidgetsDir,
    name: "livedata_text_field",
    content: liveDataTextField,
    packages: [liveData],
  ),
];

List<FileStructure> uiButtonsFiles = [
  FileStructure(
    path: buttonsWidgetsDir,
    name: "customized_button",
    content: customizedButton,
  ),
  FileStructure(
    path: buttonsWidgetsDir,
    name: "text_button",
    content: textButton,
  ),
];

List<FileStructure> uiInfiniteScrollFiles = [
  FileStructure(
    path: infiniteScrollWidgetsDir,
    name: "infinite_scroll_view",
    content: infiniteScrollView,
    packages: [animatedInfiniteScrollPagination],
  ),
  FileStructure(
    path: infiniteScrollWidgetsDir,
    name: "pagination_error_widget",
    content: paginationErrorWidget,
  ),
  FileStructure(
    path: infiniteScrollWidgetsDir,
    name: "pagination_load_widget",
    content: paginationLoaderWidget,
  ),
];

List<FileStructure> uiAnimationsFiles = [
  customizedAnimatedWidgetFile,
  scaleFadeWidgetFile,
  FileStructure(
    path: animationsWidgetsDir,
    name: "animated_column",
    content: animatedColumn,
  ),
  FileStructure(
    path: animationsWidgetsDir,
    name: "animated_gesture",
    content: animatedGesture,
  ),
  FileStructure(
    path: animationsWidgetsDir,
    name: "animated_color_widget",
    content: animatedColorWidget,
  ),
];

List<FileStructure> reusableFiles = [
  ...loaderFiles,
  ...baseLayoutFiles,
  ...uiTextFieldsFiles,
  ...uiModelsFiles,
  ...uiAnimationsFiles,
  ...uiButtonsFiles,
  ...uiInfiniteScrollFiles,
  snackbarFile,
  FileStructure(
    path: uiWidgetsDir,
    name: "tabs_view_layout",
    content: tabsViewLayout,
  ),
  FileStructure(
    path: extensionsDir,
    name: "assets_extensions",
    content: assetsExtensions,
  ),
  FileStructure(
    path: uiWidgetsDir,
    name: "text_view",
    content: textView,
  ),
];
