import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String blurWidget() => """
$widgetTitle
import ${materialPackage};
import 'dart:ui' as ui;

class BlurWidget extends StatelessWidget {
  final double width;
  final double height;
  final double blur;
  const BlurWidget({
    required this.width,
    required this.height,
    this.blur = 3,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: width,
      height: height,
      child: ClipRRect(
        child: BackdropFilter(
          filter: ui.ImageFilter.blur(sigmaX: blur, sigmaY: blur),
          child: Container(
            width: width,
            height: height,
            color: Colors.black.withOpacity(0.35),
          ),
        ),
      ),
    );
  }
}
""";

String fullScreenLoader() => """
$widgetTitle
import ${materialPackage};
import 'package:${packageName}/ui/widgets/animations/customized_animated_widget.dart';
import 'loader.dart';

class FullScreenLoader extends StatelessWidget {
  const FullScreenLoader({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return CustomizedAnimatedWidget(
      delay: const Duration(milliseconds: 0),
      duration: const Duration(milliseconds: 350),
      child: Scaffold(
        backgroundColor: Colors.black.withOpacity(0.2),
        body: const Center(child: Loader()),
      ),
    );
  }
}
""";

String loader() => """
$widgetTitle
import ${materialPackage};
import 'package:flutter_spinkit/flutter_spinkit.dart';

class Loader extends StatelessWidget {
  final double? size;
  final Color? color;
  const Loader({Key? key, this.color, this.size}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SpinKitCircle(
      color: color ?? Theme.of(context).primaryColor,
      size: size ?? MediaQuery.of(context).size.width * 0.2,
    );
  }
}
""";

String liveDataLoader() => """
import ${materialPackage};
import ${liveDataPackage};
import 'blur_widget.dart';
import 'full_screen_loader.dart';

class LoadingListenerWidget extends StatelessWidget {
  final LiveData<bool> loading;
  final double blur;
  const LoadingListenerWidget({required this.loading, this.blur = 1.8, Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LiveDataBuilder<bool>(
      data: loading,
      builder: (context, boolean) {
        return boolean
            ? Stack(
                children: [
                  BlurWidget(width: double.infinity, height: double.infinity, blur: blur),
                  const FullScreenLoader(),
                ],
              )
            : const SizedBox.shrink();
      },
    );
  }
}
""";

String progressBar() => """
$widgetTitle
import 'dart:io' show Platform;
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'loader.dart';

class AppProgressBar extends StatelessWidget {
  final double strokeWidth;
  final Color? color;
  const AppProgressBar({this.color, this.strokeWidth = 4, Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Platform.isIOS
          ? CupertinoActivityIndicator(
              color: color ?? Theme.of(context).colorScheme.primary,
            )
          : CircularProgressIndicator(
              strokeWidth: strokeWidth,
              valueColor: AlwaysStoppedAnimation<Color>(
                color ?? Theme.of(context).colorScheme.primary,
              ),
            ),
    );
  }
}

Widget textFieldLoader() => const Padding(
      padding: EdgeInsets.symmetric(horizontal: 8),
      child: Loader(size: 25),
    );

BoxConstraints textFieldLoaderConstraints() => const BoxConstraints(maxWidth: 40, maxHeight: 25);
""";

String snackBar() => """
$widgetTitle
import ${materialPackage};
import ${liveDataPackage};
import 'dart:async';
import 'package:flutter/scheduler.dart';
import 'package:${packageName}/extensions/nullable_extensions.dart';
import 'package:${packageName}/models/ui_models/ui_message.dart';

class SnackBarMessageListener extends StatefulWidget {
  final LiveData<UiMessage> uiMessage;
  const SnackBarMessageListener({required this.uiMessage, Key? key}) : super(key: key);

  @override
  State<SnackBarMessageListener> createState() => _SnackBarMessageListenerState();
}

class _SnackBarMessageListenerState extends State<SnackBarMessageListener> with StateObserver {
  @override
  Widget build(BuildContext context) {
    return const SizedBox.shrink();
  }

  @override
  FutureOr<void> registerObservers() {
    widget.uiMessage.observe(this, (value) {
      value.message?.let((it) => showSnackBar(context: context, message: it));
      value.message = null;
    });
  }
}

void showSnackBar({required BuildContext context, required String message, Function()? callback, TextStyle? textStyle}) {
  ScaffoldMessenger.of(context)
    ..hideCurrentSnackBar()
    ..showSnackBar(
      SnackBar(
        content: Text(message, style: textStyle),
        backgroundColor: Theme.of(context).colorScheme.onSurface,
        action: SnackBarAction(
          label: "Ok",
          onPressed: () {
            callback?.call();
          },
        ),
      ),
    );
}
""";
