import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String lightTheme() {
  return """
$widgetTitle
import ${materialPackage};
import '../colors/colors_night.dart';
import '../dimensions/dimensions.dart';
import '../colors/colors.dart';
import 'transitions.dart';

const fontFamily = "Poppins"; // change as you wish


/// -------------- App Theme -------------- ///
final ThemeData lightTheme = ThemeData(
  // Define the default brightness and colors.
  fontFamily: fontFamily,
  visualDensity: VisualDensity.adaptivePlatformDensity,
  brightness: Brightness.light,
  primaryColor: DesignColors.primaryColor,
  dividerColor: DesignColors.grey,
  disabledColor: DesignColors.grey,
  backgroundColor: Colors.white,
  scaffoldBackgroundColor: DesignColors.secondaryBackgroundColor,
  colorScheme: const ColorScheme(
    primary: DesignColors.primaryColor,
    secondary: DesignColors.secondaryColor,
    surface: DesignColors.primaryColor,
    background: Colors.white,
    error: DesignColors.errorColor,
    onPrimary: Colors.white,
    onSecondary: Colors.white,
    onSurface: Colors.black,
    onBackground: Color.fromRGBO(249, 249, 249, 1),
    onError: DesignColors.errorColor,
    brightness: Brightness.light,
  ),
  textTheme: textTheme(),
  elevatedButtonTheme: elevatedButtonTheme(),
  outlinedButtonTheme: outlinedButtonTheme(),
  textButtonTheme: textButtonTheme(),
  pageTransitionsTheme: pageTransitionsTheme(),
);

/// -------------- Page Transitions Animations -------------- ///
PageTransitionsTheme pageTransitionsTheme({bool darkMode = false}) => PageTransitionsTheme(
      builders: {
        TargetPlatform.android: SharedAxisTransitionBuilder(darkMode: darkMode),
        TargetPlatform.iOS: SharedAxisTransitionBuilder(darkMode: darkMode),
        TargetPlatform.windows: SharedAxisTransitionBuilder(darkMode: darkMode),
        TargetPlatform.macOS: SharedAxisTransitionBuilder(darkMode: darkMode),
        TargetPlatform.linux: SharedAxisTransitionBuilder(darkMode: darkMode),
      },
    );

/// -------------------------------------------------------- ///

/// ---------------------- Text Theme ---------------------- ///
TextTheme textTheme({bool darkMode = false}) {
  return TextTheme(
    button: const TextStyle(color: Colors.white),
    bodyText1: TextStyle(color: darkMode ? DesignColorsDark.textColor : DesignColors.textColor),
    bodyText2: TextStyle(color: darkMode ? DesignColorsDark.textColor : DesignColors.textColor),
  );
}

TextStyle outlinedButtonTextStyle({bool darkMode = false}) {
  final primaryColor = darkMode ? DesignColorsDark.primaryColor : DesignColors.primaryColor;
  return TextStyle(
    color: primaryColor,
    fontSize: 14,
    fontFamily: fontFamily,
    fontWeight: FontWeight.w500,
  );
}

TextStyle textButtonTextStyle({bool darkMode = false}) {
  final primaryColor = darkMode ? DesignColorsDark.primaryColor : DesignColors.primaryColor;
  return TextStyle(
    color: primaryColor,
    fontSize: 20,
    fontFamily: fontFamily,
    fontWeight: FontWeight.w400,
  );
}

TextStyle titleStyle(ThemeData theme) => TextStyle(
      color: theme.primaryColor,
      fontSize: normalText,
      fontWeight: FontWeight.w500,
    );

/// ----------------- [TextFormField] default styles ----------------- ///
TextStyle hintStyle() => const TextStyle(fontSize: smallText, color: DesignColors.grey);

TextStyle errorStyle() => const TextStyle(fontSize: 10, color: DesignColors.errorColor);

TextStyle labelStyle(ThemeData themeData) => const TextStyle(
      fontSize: smallText,
      color: DesignColors.grey,
      fontWeight: FontWeight.w500,
    );

/// ----------------- [TextFormField] default border ----------------- ///
OutlineInputBorder textFormFieldEnabledBorder() => OutlineInputBorder(
      borderSide: const BorderSide(color: DesignColors.grey, width: 0.8),
      borderRadius: BorderRadius.circular(textFormFieldRadius),
    );

/// ----------------- [TextFormField] enabled border ----------------- ///
OutlineInputBorder textFormFieldFocusedBorder(ThemeData themeData) => OutlineInputBorder(
      borderSide: BorderSide(color: themeData.primaryColor, width: 2),
      borderRadius: BorderRadius.circular(textFormFieldRadius),
    );

/// ----------------- [TextFormField] error border ----------------- ///
OutlineInputBorder textFormFieldErrorBorder(ThemeData themeData) => OutlineInputBorder(
      borderSide: BorderSide(color: themeData.errorColor, width: 2),
      borderRadius: BorderRadius.circular(textFormFieldRadius),
    );

/// -------------------------------------------------------- ///

/// -------------------- Buttons Theme -------------------- ///
TextButtonThemeData textButtonTheme({bool darkMode = false}) {
  final primaryColor = darkMode ? DesignColorsDark.primaryColor : DesignColors.primaryColor;
  return TextButtonThemeData(
    style: ButtonStyle(
      foregroundColor: MaterialStateProperty.all<Color>(primaryColor),
      overlayColor: MaterialStateProperty.all<Color>(Colors.transparent),
      shape: MaterialStateProperty.all<OutlinedBorder>(
        const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(Radius.circular(buttonCorner)),
        ),
      ),
      textStyle: MaterialStateProperty.all<TextStyle>(textButtonTextStyle(darkMode: darkMode)),
    ),
  );
}

OutlinedButtonThemeData outlinedButtonTheme({bool darkMode = false}) {
  final primaryColor = darkMode ? DesignColorsDark.primaryColor : DesignColors.primaryColor;
  return OutlinedButtonThemeData(
    style: ButtonStyle(
      textStyle: MaterialStateProperty.all<TextStyle>(outlinedButtonTextStyle(darkMode: darkMode)),
      foregroundColor: MaterialStateProperty.all<Color>(primaryColor),
      overlayColor: MaterialStateProperty.all<Color>(Colors.transparent),
      side: MaterialStateProperty.all<BorderSide>(BorderSide(color: primaryColor)),
      padding: MaterialStateProperty.all<EdgeInsetsGeometry>(const EdgeInsets.all(buttonPadding)),
      shape: MaterialStateProperty.all<OutlinedBorder>(
        const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(Radius.circular(buttonCorner)),
        ),
      ),
    ),
  );
}

ElevatedButtonThemeData elevatedButtonTheme({bool darkMode = false}) {
  final primaryColor = darkMode ? DesignColorsDark.primaryColor : DesignColors.primaryColor;
  return ElevatedButtonThemeData(
    style: ElevatedButton.styleFrom(
      primary: primaryColor,
      onPrimary: Colors.white,
      padding: const EdgeInsets.all(buttonPadding),
      shape: const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(buttonCorner))),
      textStyle: const TextStyle(
        color: Colors.white,
        fontSize: 20,
        fontFamily: fontFamily,
        fontWeight: FontWeight.w400,
      ),
    ),
  );
}
""";
}

String darkTheme() {
  return """
$widgetTitle
import ${materialPackage};
import '../colors/colors_night.dart';
import 'themes.dart';
import '../colors/colors.dart';

/// -------------- App Dark Theme -------------- ///
final ThemeData darkTheme = ThemeData(
  // Define the default brightness and colors.
  fontFamily: fontFamily,
  visualDensity: VisualDensity.adaptivePlatformDensity,
  brightness: Brightness.dark,
  primaryColor: DesignColorsDark.primaryColor,
  dividerColor: const Color(0xFF6D6A69),
  disabledColor: const Color(0XFFbfbfbf),
  backgroundColor: const Color.fromARGB(255, 20, 20, 20),
  scaffoldBackgroundColor: DesignColorsDark.secondaryBackgroundColor,
  colorScheme: const ColorScheme(
    primary: DesignColorsDark.primaryColor,
    secondary: DesignColorsDark.secondaryColor,
    surface: DesignColorsDark.primaryColor,
    background: Colors.black,
    error: DesignColors.errorColor,
    onPrimary: Colors.white,
    onSecondary: Colors.white,
    onSurface: Colors.white,
    onBackground: Color.fromRGBO(29, 29, 29, 1),
    onError: DesignColors.errorColor,
    brightness: Brightness.dark,
  ),
  textTheme: textTheme(darkMode: true),
  elevatedButtonTheme: elevatedButtonTheme(darkMode: true),
  outlinedButtonTheme: outlinedButtonTheme(darkMode: true),
  textButtonTheme: textButtonTheme(darkMode: true),
  pageTransitionsTheme: pageTransitionsTheme(darkMode: true),
);
""";
}
