import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String themeProvider() => """
$widgetTitle
import ${materialPackage};
import 'package:${packageName}/storage/storage.dart';
import 'package:${packageName}/ui/resources/themes/themes.dart';
import 'package:${packageName}/ui/resources/themes/themes_night.dart';

/// state management for App [ThemeMode] and [ThemeData]
class ThemeProvider with ChangeNotifier {
  ThemeMode _mode = AppStorage.themeMode();
  ThemeMode get mode => _mode;
  ThemeData get theme => mode == ThemeMode.dark ? darkTheme : lightTheme;

  void setDarkTheme(bool isDark) async {
    await AppStorage.setDarkTheme(isDark);
    _mode = isDark ? ThemeMode.dark : ThemeMode.light;
    notifyListeners();
  }
}
""";

String themeWidget() => """
$widgetTitle
import ${materialPackage};
import ${providerPackage};
import '../providers/theme_provider.dart';

class ThemeWidget extends StatelessWidget {
  /// callback which wrap your widgets in current appTheme
  final Widget Function(BuildContext context, ThemeData theme) builder;

  /// widget with state management for App [ThemeData]
  const ThemeWidget({required this.builder, Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<ThemeProvider>(
      builder: (context, value, _) => builder(context, value.theme),
    );
  }
}
""";

String baseScrollView() => """
$widgetTitle
import ${materialPackage};

const scrollPhysics = BouncingScrollPhysics();

class BaseScrollView extends StatelessWidget {
  final Widget child;
  final ScrollPhysics? physics;
  const BaseScrollView({required this.child, this.physics, Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final screenSize = MediaQuery.of(context).size;
    return LayoutBuilder(
      builder: (context, constraint) {
        return SingleChildScrollView(
          physics: physics ?? scrollPhysics,
          child: ConstrainedBox(
            // expand child
            constraints: BoxConstraints(
              minHeight: constraint.maxHeight,
              minWidth: screenSize.width,
            ),
            child: child,
          ),
        );
      },
    );
  }
}
""";

String baseScaffold() => """
$widgetTitle
import ${materialPackage};
import 'package:flutter/services.dart';
import 'package:${packageName}/extensions/theme_extensions.dart';
import 'package:${packageName}/extensions/nullable_extensions.dart';
import 'theme_widget.dart';

class BaseScaffold extends StatefulWidget {
  final Brightness? statusBarIconBrightness;
  final Widget? drawer;
  final PreferredSizeWidget? Function(BuildContext context, ThemeData theme)? appBar;
  final FloatingActionButton? floatingActionButton;
  final Color? backgroundColor;
  final Widget? bottomNavigationBar;
  final bool? resizeToAvoidBottomInset;
  final Widget Function(BuildContext context, ThemeData theme) builder;

  const BaseScaffold({
    required this.builder,
    this.appBar,
    this.drawer,
    this.floatingActionButton,
    this.statusBarIconBrightness,
    this.backgroundColor,
    this.bottomNavigationBar,
    this.resizeToAvoidBottomInset,
    Key? key,
  }) : super(key: key);

  @override
  State<BaseScaffold> createState() => BaseScaffoldState();
}

class BaseScaffoldState extends State<BaseScaffold> {
  final GlobalKey<ScaffoldState> scaffoldKey = GlobalKey();

  @override
  Widget build(BuildContext context) {
    return ThemeWidget(
      builder: (context, theme) {
        final brightness = theme.isDark() ? Brightness.light : Brightness.dark;
        return AnnotatedRegion<SystemUiOverlayStyle>(
          value: SystemUiOverlayStyle(statusBarColor: Colors.transparent, statusBarIconBrightness: widget.statusBarIconBrightness ?? brightness),
          child: Scaffold(
            key: scaffoldKey,
            extendBody: true,
            extendBodyBehindAppBar: true,
            backgroundColor: widget.backgroundColor ?? theme.scaffoldBackgroundColor,
            drawer: widget.drawer,
            body: widget.builder(context, theme),
            appBar: widget.appBar?.let((it) => it(context, theme)),
            floatingActionButton: widget.floatingActionButton,
            bottomNavigationBar: widget.bottomNavigationBar,
            resizeToAvoidBottomInset: widget.resizeToAvoidBottomInset,
          ),
        );
      },
    );
  }
}
""";
