import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String customizedButton() => """
$widgetTitle
import ${materialPackage};
import 'package:${packageName}/extensions/theme_extensions.dart';
import 'package:${packageName}/ui/core/layouts/theme_widget.dart';
import 'package:${packageName}/ui/resources/themes/themes.dart';
import '../animations/scale_fade_widget.dart';

class CustomizedButton extends StatefulWidget {
  final Widget child;
  final bool enabled;
  final Function() callback;
  final ButtonStyle? style;
  const CustomizedButton({
    required this.child,
    required this.callback,
    this.enabled = true,
    this.style,
    Key? key,
  }) : super(key: key);

  @override
  State<CustomizedButton> createState() => _CustomizedButtonState();
}

class _CustomizedButtonState extends State<CustomizedButton> {
  final GlobalKey<ScaleFadeWidgetState> globalKey = GlobalKey<ScaleFadeWidgetState>();
  final GlobalKey<ScaleFadeWidgetState> childKey = GlobalKey<ScaleFadeWidgetState>();
  @override
  Widget build(BuildContext context) {
    return ThemeWidget(
      builder: (context, theme) {
        return ScaleFadeWidget(
          key: globalKey,
          type: AnimationType.scale,
          scaleValue: 0.015,
          duration: const Duration(milliseconds: 200),
          child: ElevatedButton(
            onPressed: () async {
              if (!widget.enabled) return;
              globalKey.currentState?.animate();
              childKey.currentState?.animate(callback: widget.callback);
            },
            style: (widget.style ?? elevatedButtonTheme(darkMode: theme.isDark()).style)?.copyWith(
              overlayColor: MaterialStateProperty.all(Colors.transparent),
              backgroundColor: MaterialStateProperty.all(widget.enabled ? theme.primaryColor : Colors.grey.shade400),
            ),
            child: ScaleFadeWidget(
              duration: const Duration(milliseconds: 200),
              type: AnimationType.fade,
              fadeValue: 0.5,
              key: childKey,
              child: widget.child,
            ),
          ),
        );
      },
    );
  }
}
""";

String textButton() => """
$widgetTitle
import ${materialPackage};
import 'package:${packageName}/ui/resources/themes/themes.dart';
import '../animations/scale_fade_widget.dart';

class AppTextButton extends StatefulWidget {
  final String text;
  final TextStyle? textStyle;
  final bool enabled;
  final Function() callback;

  const AppTextButton({
    required this.text,
    this.textStyle,
    required this.callback,
    this.enabled = true,
    Key? key,
  }) : super(key: key);

  @override
  State<AppTextButton> createState() => _AppTextButtonState();
}

class _AppTextButtonState extends State<AppTextButton> {
  final GlobalKey<ScaleFadeWidgetState> globalKey = GlobalKey<ScaleFadeWidgetState>();

  @override
  Widget build(BuildContext context) {
    final textStyle = widget.textStyle ?? textButtonTextStyle(darkMode: Theme.of(context).brightness == Brightness.dark);
    final buttonTextStyle = widget.enabled ? textStyle : textStyle.copyWith(color: Colors.grey.shade400);
    return ScaleFadeWidget(
      key: globalKey,
      duration: const Duration(milliseconds: 150),
      child: TextButton(
        child: Text(widget.text, style: buttonTextStyle),
        onPressed: () async {
          if (!widget.enabled) return;
          globalKey.currentState?.animate(callback: widget.callback);
        },
      ),
    );
  }
}
""";
