import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String nullableExtensions() => """
$widgetTitle

extension ObjectExt<T> on T {
  /// equivalent to Kotlin's val?.let { it -> someLogic }
  ///
  /// usage:
  ///
  /// int? x = null; x?.let((it) => someLogic)
  R let<R>(R Function(T it) op) => op(this);
}

extension ListUtils<T> on List<T?>? {
  /// return empty list if the list is null
  List<T> orEmpty() {
    return filterNotNull();
  }

  /// remove null items from list
  List<T> filterNotNull() {
    final List<T> list = [];
    this?.forEach((element) {
      if (element != null) list.add(element);
    });
    return list;
  }
}
""";

String assetsExtensions() => """
$widgetTitle

extension AssetsUtils on String {
  String get svgAsset => 'assets/svgs/\$this.svg';
  String get pngAsset => 'assets/images/\$this.png';
  String get webpAsset => 'assets/images/\$this.webp';
  String get gifAsset => 'assets/gifs/\$this.gif';
}
""";

String themeExtensions() => """
$widgetTitle
import ${materialPackage};
import 'package:${packageName}/storage/storage.dart';

extension ThemeUtils on ThemeData {
  /// get current theme textColor
  Color? get textColor => textTheme.bodyText1?.color;

  /// if applied theme is dark theme
  bool isDark() => AppStorage.themeMode() == ThemeMode.dark;
}
""";
String formzExtensions() => """
$widgetTitle
import ${liveDataPackage};
import ${formzPackage};

extension LiveDataInputUtils on LiveData<FormzInput> {
  bool isNotEmpty() {
    return value.value.isNotEmpty;
  }

  String inputValue() {
    return value.value;
  }
}
""";
