import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String infiniteScrollView() => """
$widgetTitle
import 'package:animated_infinite_scroll_pagination/animated_infinite_scroll_pagination.dart';
import ${materialPackage};
import 'package:${packageName}/config/singleton_config.dart';
import 'package:${packageName}/ui/core/layouts/base_scroll_view.dart';
import 'pagination_load_widget.dart';
import 'pagination_error_widget.dart';
import '../loaders/loader.dart';
import '../animations/customized_animated_widget.dart';

/// [VM] a view model extends [PaginationViewModel]
/// [T] the Model of item
class InfiniteScrollView<VM extends PaginationViewModel<T>, T extends Object> extends StatelessWidget {
  /// [itemBuilder] callback return [Widget] with item [T].
  final Widget Function(T) itemBuilder;

  /// [onRefresh] callback called when user swipe to refresh list.
  final Function()? onRefresh;

  /// [retry] callback called when pagination request faild and user press on [PaginationErrorWidget].
  final Function()? retry;
  const InfiniteScrollView({
    required this.itemBuilder,
    this.onRefresh,
    this.retry,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedInfiniteScrollView<T>(
      viewModel: getSingleton<VM>(),
      refreshIndicator: true,
      loadingWidget: const CustomizedAnimatedWidget(child: Loader()),
      footerLoadingWidget: const PaginationLoaderWidget(),
      errorWidget: const PaginationErrorWidget(),
      physics: scrollPhysics,
      onRefresh: onRefresh,
      itemBuilder: (item) => CustomizedAnimatedWidget(child: itemBuilder.call(item)),
    );
  }
}
""";

String paginationErrorWidget() => """
$widgetTitle
import ${materialPackage};
import '../animations/customized_animated_widget.dart';
import '../buttons/text_button.dart';

class PaginationErrorWidget extends StatelessWidget {
  final Function()? retry;
  const PaginationErrorWidget({this.retry, Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return CustomizedAnimatedWidget(
      child: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 10),
        child: AppTextButton(
          text: "Error try again",
          callback: () {
            retry?.call();
          },
        ),
      ),
    );
  }
}
""";

String paginationLoaderWidget() => """
$widgetTitle
import ${materialPackage};
import '../animations/customized_animated_widget.dart';
import '../loaders/progress_bar.dart';

class PaginationLoaderWidget extends StatelessWidget {
  const PaginationLoaderWidget({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const CustomizedAnimatedWidget(
      child: SizedBox(width: 30, height: 30, child: AppProgressBar(strokeWidth: 2.5)),
    );
  }
}
""";
