import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String deviceScreenType() => """
$widgetTitle
/// more info for responsive: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter
enum DeviceScreenType { mobile, tablet, desktop }
""";

String orientationLayout() => """
$widgetTitle
/// more info for responsive: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter
import ${materialPackage};

class OrientationLayout extends StatelessWidget {
  final Widget unsetWidget = const Scaffold(body: Center(child: Text('OrientationLayout')));
  final Widget? landscape;
  final Widget? portrait;
  const OrientationLayout({
    Key? key,
    this.landscape,
    this.portrait,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    var orientation = MediaQuery.of(context).orientation;
    if (orientation == Orientation.portrait) {
      return portrait ?? landscape ?? unsetWidget;
    }
    return landscape ?? unsetWidget;
  }
}
""";

String responsiveBuilder() => """
$widgetTitle
/// more info for responsive: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter
import ${materialPackage};
import 'responsive_utils.dart';
import 'sizing_information.dart';

class ResponsiveBuilder extends StatelessWidget {
  final Widget Function(
    BuildContext context,
    SizingInformation sizingInformation,
  ) builder;
  const ResponsiveBuilder({Key? key, required this.builder}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, boxConstraints) {
        var mediaQuery = MediaQuery.of(context);
        var sizingInformation = SizingInformation(
          deviceScreenType: getDeviceType(mediaQuery),
          screenSize: mediaQuery.size,
          localWidgetSize: Size(boxConstraints.maxWidth, boxConstraints.maxHeight),
        );
        return builder(context, sizingInformation);
      },
    );
  }
}
""";

String responsiveUtils() => """
$widgetTitle
/// more info for responsive: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter
import 'package:flutter/widgets.dart';
import 'device_screen_type_enums.dart';

DeviceScreenType getDeviceType(MediaQueryData mediaQuery) {
  double deviceWidth = mediaQuery.size.shortestSide;

  if (deviceWidth > 950) {
    return DeviceScreenType.desktop;
  }

  if (deviceWidth > 600) {
    return DeviceScreenType.tablet;
  }

  return DeviceScreenType.mobile;
}
""";

String screenTypeLayout() => """
$widgetTitle
/// more info for responsive: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter
import ${materialPackage};
import 'device_screen_type_enums.dart';
import 'responsive_builder.dart';

const defaultWidget = Center(child: Text("Welcome"));

class ScreenTypeLayout extends StatelessWidget {
  final Widget? mobile;
  final Widget? tablet;
  final Widget? desktop;

  const ScreenTypeLayout({Key? key, this.mobile, this.tablet, this.desktop}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ResponsiveBuilder(builder: (context, sizingInformation) {
      // If sizing indicates mobile and we have a mobile widget then return
      if (sizingInformation.deviceScreenType == DeviceScreenType.mobile) {
        return mobile ?? tablet ?? desktop ?? defaultWidget;
      }

      // If sizing indicates tablet and we have a tablet widget then return
      if (sizingInformation.deviceScreenType == DeviceScreenType.tablet) {
        return tablet ?? mobile ?? desktop ?? defaultWidget;
      }

      // If sizing indicates desktop and we have a desktop widget then return
      if (sizingInformation.deviceScreenType == DeviceScreenType.desktop) {
        return desktop ?? tablet ?? mobile ?? defaultWidget;
      }

      // Return tablet layout if nothing else is supplied
      return mobile ?? tablet ?? desktop ?? defaultWidget;
    });
  }
}
""";

String sizingInformation() => """
$widgetTitle
/// more info for responsive: https://www.filledstacks.com/post/building-a-responsive-ui-architecture-in-flutter
import ${materialPackage};
import 'device_screen_type_enums.dart';

class SizingInformation {
  final DeviceScreenType? deviceScreenType;
  final Size? screenSize;
  final Size? localWidgetSize;

  SizingInformation({
    this.deviceScreenType,
    this.screenSize,
    this.localWidgetSize,
  });
}
""";
