import 'package:xflutter_cli/src/commands/i_flags.dart';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String screenWidget({
  required String className,
  required String rootPath,
  bool isParent = false,
  String? child,
}) {
  final theme = flags.contains(CliFlags.theme);
  final responsive = flags.contains(CliFlags.responsive);
  final singleton = flags.contains(CliFlags.singleton);
  return """
$widgetTitle
import ${materialPackage};
${singleton ? "" : 'import "${isParent ? '.' : '..'}/viewmodels/${rootPath}_viewmodel.dart";'}
${theme ? """
import 'package:${packageName}/ui/core/layouts/base_scaffold.dart';
import 'package:${packageName}/ui/core/layouts/theme_widget.dart';
""" : ""}
${isParent ? """
import 'package:${packageName}/ui/widgets/loaders/live_data_loader.dart';
import 'package:${packageName}/ui/widgets/snackbar.dart';
""" : ""}

${singleton ? statelessWidget(className) : statefulWidget(className, variables: "final viewModel = ${className}ViewModel();")}

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        ${theme ? '''BaseScaffold(
          builder: (context, theme) {
           return ${isParent ? responsiveWidget(responsive, className: className) : themeWrapper(theme, child: child)};
            },
          )''' : '''Scaffold(
            body: ${isParent ? responsiveWidget(responsive, className: className) : themeWrapper(theme, child: child)},
          )'''},
            ${isParent ? """
LoadingListenerWidget(loading: viewModel.params.loading),
            SnackBarMessageListener(uiMessage: viewModel.params.uiMessage),
""" : ""}
        ],
      );
    }
}
""";
}

String singletonInstance({
  required String className,
  required String rootPath,
}) {
  final singleton = flags.contains(CliFlags.singleton);
  if (!singleton) {
    return screenWidget(className: className, isParent: true, rootPath: rootPath);
  }

  final theme = flags.contains(CliFlags.theme);
  final responsive = flags.contains(CliFlags.responsive);
  final tablet = flags.contains(CliFlags.tablet);
  return """
$widgetTitle
import ${materialPackage};
import "package:${packageName}/ui/widgets/singleton/singleton_instance.dart";
import "./viewmodels/${rootPath}_viewmodel.dart";
import 'package:${packageName}/ui/widgets/loaders/live_data_loader.dart';
import 'package:${packageName}/ui/widgets/snackbar.dart';
${responsive ? """
import 'package:${packageName}/ui/core/responsive/screen_type_layout.dart';
import 'mobile/${rootPath}_mobile_screen.dart';""" : ""}
${tablet ? """
import 'tablet/${rootPath}_tablet_screen.dart';""" : ""}
${theme ? """
import 'package:${packageName}/ui/core/layouts/base_scaffold.dart';
${!singleton ? "import 'package:${packageName}/ui/core/layouts/theme_widget.dart';" : ""} """ : ""}

class ${className}Screen extends StatelessWidget {
  const ${className}Screen({Key? key}) : super(key: key);
  
  @override
  Widget build(BuildContext context) {
    return SingletonInstance<${className}ViewModel>(
      instance: ${className}ViewModel(),
      initState: (viewModel) {},
      registerObservers: (lifeCycle, viewModel) {},
      dispose: (viewModel) {},
      builder: (viewModel) {
        return Stack(
          children: [
            ${theme ? '''BaseScaffold(
          builder: (context, theme) {
            return ${responsiveWidget(responsive, className: className)};
            },
          )''' : '''Scaffold(
            body: ${responsiveWidget(responsive, className: className)},
          )'''},
            LoadingListenerWidget(loading: viewModel.params.loading),
            SnackBarMessageListener(uiMessage: viewModel.params.uiMessage),
          ],
        );
      },
    );
  }
}
""";
}

String singletonWidget({
  required String className,
  required String rootPath,
  required String singletonName,
  String? child,
}) {
  final singleton = flags.contains(CliFlags.singleton);
  if (!singleton) {
    return screenWidget(className: className, rootPath: rootPath);
  }

  final theme = flags.contains(CliFlags.theme);
  return """
$widgetTitle
import ${materialPackage};
import "package:${packageName}/ui/widgets/singleton/singleton_widget.dart";
import "../viewmodels/${rootPath}_viewmodel.dart";
${theme ? """
import 'package:${packageName}/ui/core/layouts/theme_widget.dart';
""" : ""}

class ${className}Screen extends StatelessWidget {
  const ${className}Screen({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return SingletonWidget<${singletonName}>(
      builder: (viewModel) {
       return ${themeWrapper(theme, child: child)};
      },
    );
  }
}
""";
}

String themeWrapper(bool theme, {String? child}) {
  final widget = child ?? " Container()";
  if (theme) {
    return """
 ThemeWidget(
          builder: (context, theme) {
          return $widget;
          },
        )""";
  }
  return widget;
}

String responsiveWidget(bool responsive, {required String className, String child = " Container()"}) {
  final tablet = flags.contains(CliFlags.tablet);
  if (responsive) {
    return """
 const ScreenTypeLayout(mobile: ${className}MobileScreen() ${tablet ? ", tablet: ${className}TabletScreen()," : ''})""";
  }
  return child;
}

String statelessWidget(String className) {
  return """class ${className}Screen extends StatelessWidget {
  const ${className}Screen({Key? key}) : super(key: key);""";
}

String statefulWidget(String className, {String? variables, String? initState}) {
  return """class ${className}Screen extends StatefulWidget {
  const ${className}Screen({Key? key}) : super(key: key);

  @override
  State<${className}Screen> createState() => _${className}ScreenState();
}

class _${className}ScreenState extends State<${className}Screen> {
  ${variables != null ? variables : ""}

  @override
  void initState() {
      ${initState != null ? variables : ""}
    super.initState();
  }
  """;
}
