import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String singletonWidget() => """
$widgetTitle
import ${materialPackage};
import 'singleton_instance.dart';
import 'package:${packageName}/config/singleton_config.dart';

class SingletonWidget<T extends Object> extends StatefulWidget {
  /// callback which wrap your widgets with the same global instance you registered in [SingletonInstance].
  final Widget Function(T singleton) builder;

  /// wrap child widgets with the same global instance which registered in [SingletonInstance].
  /// 
  /// avoid passing variables from parent to children.
  /// 
  /// just create instance in the parent in use this instance in the children widgets.
  const SingletonWidget({required this.builder, Key? key}) : super(key: key);

  @override
  State<SingletonWidget> createState() => _SingletonWidgetState<T>();
}

class _SingletonWidgetState<T extends Object> extends State<SingletonWidget<T>> {

  @override
  Widget build(BuildContext context) {
    return widget.builder.call(getSingleton<T>());
  }
}
""";

String singletonInstance() => """
$widgetTitle
import 'dart:async';
import ${materialPackage};
import ${liveDataPackage};
import 'package:${packageName}/config/singleton_config.dart';
import 'singleton_widget.dart';

class SingletonInstance<T extends Object> extends StatefulWidget {
  /// new instance from you class.
  final T instance;

  /// callback which wrap your widgets with the global instance.
  final Widget Function(T singleton) builder;

  /// callback when widget initialized.
  final Function(T)? initState;

  /// callback when widget destroyed.
  final Function(T)? dispose;

  /// observe on live data variables
  final Function(StateObserver<StatefulWidget>, T)? registerObservers;

  /// Create a global instance when widget state initialized,
  /// and remove the instance when widget destroyed.
  ///
  /// this make you can access the same instance from your widget children without pass any variables.
  ///
  /// just wrap your child widget in [SingletonWidget].
  const SingletonInstance({
    required this.instance,
    required this.builder,
    this.initState,
    this.dispose,
    this.registerObservers,
    Key? key,
  }) : super(key: key);

  @override
  State<SingletonInstance<T>> createState() => _SingletonInstanceState<T>();
}

class _SingletonInstanceState<T extends Object> extends State<SingletonInstance<T>> with StateObserver {
  T get instance => getSingleton<T>();

  @override
  void initState() {
    registerSingleton<T>(widget.instance);
    widget.initState?.call(instance);
    super.initState();
  }

  @override
  void dispose() {
    widget.dispose?.call(instance);
    unregisterSingleton<T>();
    super.dispose();
  }

  @override
  FutureOr<void> registerObservers() {
    widget.registerObservers?.call(this, instance);
  }

  @override
  Widget build(BuildContext context) {
    return widget.builder.call(instance);
  }
}
""";
