import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/structure/packages.dart';

String formzText = "FormzText";
String formzEmail = "FormzEmail";
String formzPassword = "FormzPassword";
String formzConfirmPassword = "FormzConfirmPassword";

String textField() => """
$widgetTitle
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:${packageName}/ui/core/layouts/theme_widget.dart';
import 'package:${packageName}/ui/resources/themes/themes.dart';
import 'package:${packageName}/ui/resources/colors/colors.dart';

class CustomizedTextFormField extends StatefulWidget {
  final String? Function(String?)? validator;
  final String? labelText;
  final String? hintText;
  final String? errorText;
  final TextEditingController? controller;
  final TextInputAction textInputAction;
  final TextInputType? keyboardType;
  final int debounceDuration;
  final TextAlign textAlign;
  final TextStyle? textStyle;
  final Function(String)? onTextChanged;
  final bool editable;
  final bool obscureText;
  final int maxLines;
  final int minLines;
  final FocusNode? focusNode;
  final bool errorOnFocus;
  final Widget? prefix;
  final Widget? suffix;
  final Widget? prefixIcon;
  final Widget? suffixIcon;
  final BoxConstraints? prefixIconConstraints;
  final BoxConstraints? suffixIconConstraints;
  final Function? onEditingComplete;
  const CustomizedTextFormField({
    this.labelText,
    this.focusNode,
    this.hintText,
    this.errorText,
    this.controller,
    this.textInputAction = TextInputAction.next,
    this.keyboardType,
    this.textAlign = TextAlign.start,
    this.debounceDuration = 0,
    this.textStyle,
    this.onTextChanged,
    this.validator,
    this.editable = true,
    this.obscureText = false,
    this.maxLines = 1,
    this.minLines = 1,
    this.errorOnFocus = false,
    this.prefix,
    this.suffix,
    this.prefixIcon,
    this.suffixIcon,
    this.prefixIconConstraints,
    this.suffixIconConstraints,
    this.onEditingComplete,
    Key? key,
  }) : super(key: key);

  @override
  State<CustomizedTextFormField> createState() => CustomizedTextFormFieldState();
}

class CustomizedTextFormFieldState extends State<CustomizedTextFormField> {
  final GlobalKey<FormState> formKey = GlobalKey();
  final FocusNode _focusNode = FocusNode();
  FocusNode get focusNode => widget.focusNode ?? _focusNode;
  Timer? _debounce;

  @override
  void initState() {
    super.initState();
    focusNode.addListener(() {
      setState(() {});
    });
  }

  @override
  Widget build(BuildContext context) {
    return ThemeWidget(
      builder: (context, themeData) {
        bool hasError = (focusNode.hasFocus && widget.errorOnFocus) || !focusNode.hasFocus;
        Color labelColor() {
          if (hasError && widget.errorText != null) {
            return DesignColors.errorColor;
          } else if (focusNode.hasFocus) {
            return themeData.primaryColor;
          } else {
            return DesignColors.grey;
          }
        }

        return AbsorbPointer(
          absorbing: !widget.editable,
          child: TextFormField(
            key: formKey,
            focusNode: focusNode,
            controller: widget.controller,
            textAlign: widget.textAlign,
            textInputAction: widget.textInputAction,
            keyboardType: widget.keyboardType,
            style: widget.textStyle ?? themeData.textTheme.bodyText1,
            obscureText: widget.obscureText,
            autocorrect: false,
            maxLines: widget.maxLines,
            minLines: widget.minLines,
            autofocus: false,
            keyboardAppearance: Brightness.dark,
            validator: widget.validator,
            decoration: InputDecoration(
              labelText: widget.labelText,
              hintText: widget.hintText,
              errorText: hasError ? widget.errorText : null,
              filled: true,
              fillColor: themeData.backgroundColor,
              enabled: widget.editable,
              hintStyle: hintStyle(),
              labelStyle: labelStyle(themeData).copyWith(color: labelColor()),
              errorStyle: errorStyle(),
              enabledBorder: textFormFieldEnabledBorder(),
              focusedBorder: textFormFieldFocusedBorder(themeData),
              errorBorder: textFormFieldErrorBorder(themeData),
              focusedErrorBorder: textFormFieldErrorBorder(themeData),
              disabledBorder: textFormFieldEnabledBorder(),
              prefix: widget.prefix,
              prefixIcon: widget.prefixIcon,
              prefixIconConstraints: widget.prefixIconConstraints,
              suffix: widget.suffix,
              suffixIcon: widget.suffixIcon,
              suffixIconConstraints: widget.suffixIconConstraints,
              contentPadding: const EdgeInsets.all(12),
            ),
            onChanged: (val) {
              if (_debounce?.isActive ?? false) _debounce?.cancel();
              _debounce = Timer(Duration(milliseconds: widget.debounceDuration), () {
                widget.validator?.call(val);
                widget.onTextChanged?.call(val);
              });
            },
            onFieldSubmitted: (val) {
              Form.of(context)?.validate();
            },
            onSaved: (val) {
              Form.of(context)?.validate();
            },
            onEditingComplete: () {
              if (widget.onEditingComplete != null) {
                widget.onEditingComplete?.call();
              } else {
                // go to next textField
                try {
                  Form.of(context)?.validate();
                  FocusScope.of(context).nextFocus();
                } catch (_) {}
              }
            },
          ),
        );
      },
    );
  }
}
""";

String liveDataTextField() => """
$widgetTitle
import 'package:flutter/material.dart';
import 'package:flutterx_live_data/flutterx_live_data.dart';
import 'package:formz/formz.dart';
import 'package:${packageName}/extensions/formz_extensions.dart';
import 'customized_text_form_field.dart';

class LiveDataTextField<T extends FormzInput> extends StatefulWidget {
  final String? label;
  final MutableLiveData<T> liveData;
  final Function(MutableLiveData<T>, String) onTextChanged;
  final TextInputType? keyboardType;
  final TextInputAction textInputAction;
  final int maxLines;
  final Widget? prefixIcon;
  final Widget? suffixIcon;
  final BoxConstraints? prefixIconConstraints;
  final BoxConstraints? suffixIconConstraints;
  final Function? onEditingComplete;
  final FocusNode? focusNode;
  final bool editable;
  final TextStyle? textStyle;
  final String? Function()? errorText;
  final bool obscureText;
  const LiveDataTextField({
    this.label,
    required this.liveData,
    required this.onTextChanged,
    this.keyboardType,
    this.textInputAction = TextInputAction.next,
    this.maxLines = 1,
    this.prefixIcon,
    this.suffixIcon,
    this.prefixIconConstraints,
    this.suffixIconConstraints,
    this.onEditingComplete,
    this.focusNode,
    this.editable = true,
    this.textStyle,
    this.errorText,
    this.obscureText = false,
    Key? key,
  }) : super(key: key);

  @override
  State<LiveDataTextField<T>> createState() => _LiveDataTextFieldState<T>();
}

class _LiveDataTextFieldState<T extends FormzInput> extends State<LiveDataTextField<T>> {
  final key = DateTime.now().toString();
  final TextEditingController textController = TextEditingController();
  final FocusNode _focusNode = FocusNode();
  FocusNode get focusNode => widget.focusNode ?? _focusNode;

  @override
  void initState() {
    // check if liveData has a value, this used when user fill the textField
    // and navigate to other screen and back
    if (widget.liveData.isNotEmpty()) {
      textController.text = widget.liveData.inputValue();
    }
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return LiveDataBuilder<T>(
      data: widget.liveData,
      builder: (context, textInput) {
        if (textInput.value.toString().isEmpty) {
          // when clear params -> clear the textField
          textController.text = "";
        }

        return CustomizedTextFormField(
          key: Key(key),
          labelText: widget.label,
          controller: textController,
          keyboardType: widget.keyboardType,
          textInputAction: widget.textInputAction,
          maxLines: widget.maxLines,
          focusNode: focusNode,
          onTextChanged: (value) => widget.onTextChanged(widget.liveData, value),
          prefixIcon: widget.prefixIcon,
          suffixIcon: widget.suffixIcon,
          prefixIconConstraints: widget.prefixIconConstraints,
          suffixIconConstraints: widget.suffixIconConstraints,
          onEditingComplete: widget.onEditingComplete,
          editable: widget.editable,
          textStyle: widget.textStyle,
          errorText: widget.errorText?.call(),
          obscureText: widget.obscureText,
        );
      },
    );
  }
}
""";

String formzTextInput() => """
$widgetTitle
import 'package:formz/formz.dart';

enum InputError { empty, invalid }

class ${formzText} extends FormzInput<String, InputError> {
  const ${formzText}.pure([String value = '']) : super.pure(value);
  const ${formzText}.dirty([String value = '']) : super.dirty(value);

  @override
  InputError? validator(String value) {
    /// validate textField value.
    return null;
  }
}

extension Explanation on InputError {
  String? get message {
    switch (this) {
      // return your message depending on InputError.type
      default:
        return null;
    }
  }
}
""";

String formzEmailInput() => """
$widgetTitle
import 'package:formz/formz.dart';

enum EmailValidationError { invalid }

class ${formzEmail} extends FormzInput<String, EmailValidationError> {
  const ${formzEmail}.pure([String value = '']) : super.pure(value);
  const ${formzEmail}.dirty([String value = '']) : super.dirty(value);

  static final RegExp _emailRegExp = RegExp(
    r'^[a-zA-Z0-9.!#\$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*\$',
  );

  @override
  EmailValidationError? validator(String value) {
    if (value.trim().isEmpty) return null;
    return _emailRegExp.hasMatch(value) ? null : EmailValidationError.invalid;
  }
}

extension Explanation on EmailValidationError {
  String? get message {
    switch (this) {
      case EmailValidationError.invalid:
        return "Email not valid";
      default:
        return null;
    }
  }
}
""";

String formzPasswordInput() => """
$widgetTitle
import 'package:formz/formz.dart';

enum PasswordValidationError { invalid, empty }

class ${formzPassword} extends FormzInput<String, PasswordValidationError> {
  const ${formzPassword}.pure() : super.pure('');
  const ${formzPassword}.dirty([String value = '']) : super.dirty(value);

  @override
  PasswordValidationError? validator(String value) {
    if (value.isEmpty) {
      return PasswordValidationError.empty;
    }
    // TODO: validate you password
    return null;
  }
}

extension Explanation on PasswordValidationError {
  String? get message {
    switch (this) {
      case PasswordValidationError.invalid:
        return "Password invalid";
      default:
        return null;
    }
  }
}
""";

String formzConfirmPasswordInput() => """
$widgetTitle
import 'package:formz/formz.dart';

enum ConfirmedPasswordValidationError {
  invalid,
  mismatch,
}

class ${formzConfirmPassword} extends FormzInput<String, ConfirmedPasswordValidationError> {
  final String password;

  const ${formzConfirmPassword}.pure({this.password = ''}) : super.pure('');
  const ${formzConfirmPassword}.dirty({required this.password, String value = ''}) : super.dirty(value);

  @override
  ConfirmedPasswordValidationError? validator(String value) {
    if (value.isEmpty) {
      return ConfirmedPasswordValidationError.invalid;
    }
    return password == value ? null : ConfirmedPasswordValidationError.mismatch;
  }
}

extension Explanation on ConfirmedPasswordValidationError {
  String? get message {
    switch (this) {
      case ConfirmedPasswordValidationError.mismatch:
        return 'Password not confirmed';
      default:
        return null;
    }
  }
}
""";
