import 'package:args/args.dart';
import 'package:xflutter_cli/src/commands/i_terminal.dart';
import 'package:xflutter_cli/src/commands/runner/generate_command.dart';
import 'package:xflutter_cli/src/commands/runner/help_command.dart';
import 'package:xflutter_cli/src/commands/runner/init_command.dart';
import 'package:xflutter_cli/src/extensions/dynamic_extensions.dart';
import 'package:xflutter_cli/src/structure/packages.dart';
import 'package:xflutter_cli/src/structure/structure_paths.dart';

import 'commands/i_command.dart';
import 'commands/i_flags.dart';
import 'commands/i_options.dart';

class XFlutterCommandRunner {
  void run(List<String> arguments) async {
    final argParser = ArgParser();
    cliCommands.forEach((key, value) {
      argParser.addCommand(key);
    });

    screenFlags.forEach((key, value) {
      argParser.addFlag(key);
    });

    cliOptions.forEach((key, value) {
      argParser.addOption(key);
    });

    final res = argParser.parse(arguments);
    cliArguments = res.arguments;
    if (res.command != null && res.command?.name != null) {
      ICommand? command;
      final commandName = res.command?.name;

      if (commandName != CliCommands.help.name) {
        if (!"${basePath}/main.dart".asFile().existsSync() || !"pubspec.yaml".asFile().existsSync()) {
          messageAndExit(message: "Cannot find flutter project in this path\nbe sure you in correct path.");
        }
      }

      await initPubspec();
      if (commandName == CliCommands.help.name) {
        command = HelpCommand();
      } else if (commandName == CliCommands.generate.name) {
        final path = fetchPath();
        command = GenerateCommand(path);
      } else if (commandName == CliCommands.init.name) {
        command = InitCommand();
      }
      if (command != null) {
        getScreenFlags();
        command.execute();
      } else {
        messageAndExit();
      }
    } else {
      messageAndExit();
    }
  }
}
